/////////////////////////////////////////////////////////////////
// CLClab306 04.30.2024 Exercise Assistant a.k.a. Treadmill Buddy  
/////////////////////////////////////////////////////////////////
// uses LILYGO ESP32-T-Display-S3
/////////////////////////////////////////////////////////////////
//  U8g2 fonts--https://github.com/olikraus/u8g2/wiki/fntlistall
///////////////////////////////////////////////////////////////// 

#include "Arduino.h"
#include <U8g2lib.h>
#include <Arduino_GFX_Library.h>      // used version 1.4.6
#include "pin_config.h"
#include "Workouts.h"

#if ESP_IDF_VERSION >= ESP_IDF_VERSION_VAL(5,0,0)
#error  "The current version is not supported for the time being, please use a version below Arduino ESP32 3.0"
#endif

#define GFX_DEV_DEVICE LILYGO_T_DISPLAY_S3
#define GFX_EXTRA_PRE_INIT()              \
    {                                     \
        pinMode(15 /* PWD */, OUTPUT);    \
        digitalWrite(15 /* PWD */, HIGH); \
    }
#define GFX_BL 38
Arduino_DataBus *bus = new Arduino_ESP32PAR8Q(
    7 /* DC */, 6 /* CS */, 8 /* WR */, 9 /* RD */,
    39 /* D0 */, 40 /* D1 */, 41 /* D2 */, 42 /* D3 */, 45 /* D4 */, 46 /* D5 */, 47 /* D6 */, 48 /* D7 */);
Arduino_GFX *gfx = new Arduino_ST7789(bus, 5 /* RST */, 1 /* rotation */, true /* IPS */, 170 /* width */, 320 /* height */, 35 /* col offset 1 */, 0 /* row offset 1 */, 35 /* col offset 2 */, 0 /* row offset 2 */);

int32_t w, h, n, n1, cx, cy, cx1, cy1, cn, cn1;
uint8_t tsa, tsb, tsc, ds;

const int SessionStart = 14;       // LILYGO T-Display ESP32-S3 built-in buttons
const int SelectWorkout = 0;
int onFLG = 0;
int numWorkouts = 7;    //total number of workouts listed in "Workouts.h"--adjust as needed
int workout = 0;
int clockSec = 0;
int clockMin = 0;
int numSteps = 0;
int STEP = 0;
int DUR = 0;
String LBL = "";
String INTERVAL = "";
String SPD = "";
String INC = "";
String INTERVAL1 = "";
String SPD1 = "";
String INC1 = "";
unsigned long  nowTime = 0;
unsigned long  lastTime = 0;
unsigned long  criterionTime = 1000;      // check timer every second
unsigned long  waitTime = 0;
unsigned long  last_waitTime = 0;
unsigned long  criterionWait = 60 * 1000;      // wait time until sleep (.001 sec)

void IRAM_ATTR SessionStartInterrupt() 
  {
    detachInterrupt(digitalPinToInterrupt(SessionStart));
    detachInterrupt(digitalPinToInterrupt(SelectWorkout));  //disable the workout-select button after the session has started
    onFLG = !onFLG;
    nowTime = lastTime = millis();
    delay(500);
    attachInterrupt(digitalPinToInterrupt(SessionStart), SessionStartInterrupt, RISING);
  }

void IRAM_ATTR SelectWorkoutInterrupt() 
  {
    detachInterrupt(digitalPinToInterrupt(SelectWorkout));
    workout++;
    if(workout == numWorkouts)
      {
        workout = 0;
      }
    LBL = METs[workout][0][0];
    numSteps = METs[workout][0][1].toInt();
    STEP = 1;
    gfx->fillScreen(BLACK);
    showMETlabel();
    showTreadLabels();
    updateLevel(STEP);
    updateTime(); 
    delay(500);
    attachInterrupt(digitalPinToInterrupt(SelectWorkout), SelectWorkoutInterrupt, RISING);
  }

void setup() 
  {
    Serial.begin(115200);
    pinMode(SessionStart, INPUT_PULLUP);
    pinMode(SelectWorkout, INPUT_PULLUP);
    numWorkouts = 7;                      //total number of workouts listed in "Workouts.h"--adjust as needed
    workout = 0; 
    waitTime = last_waitTime = millis();
    LBL = METs[workout][0][0];
    numSteps = METs[0][0][1].toInt();     //default to 1st workout in "Workouts.h"
    STEP = 1;
    // initialize display
    pinMode(PIN_POWER_ON, OUTPUT);
    digitalWrite(PIN_POWER_ON, HIGH);
    ledcSetup(0, 2000, 8);
    ledcAttachPin(PIN_LCD_BL, 0);
    ledcWrite(0, 255);              // Screen brightness can be modified by adjusting this parameter. (0-255)
    gfx->begin();
    gfx->setRotation(1);
    gfx->fillScreen(BLACK);
    updateLevel(STEP);
    showMETlabel();
    showTreadLabels();
    updateTime();
    randomSeed(analogRead(12));
    attachInterrupt(digitalPinToInterrupt(SessionStart), SessionStartInterrupt, RISING);
    attachInterrupt(digitalPinToInterrupt(SelectWorkout), SelectWorkoutInterrupt, RISING);
  }

void loop() 
  {      
    if(onFLG)
      {
        nowTime = millis();
        if((nowTime - lastTime) >= criterionTime)
          {
            lastTime = nowTime;
            clockSec++;
            if(clockSec == 60)                 // set to 60 sec
              {
                clockSec = 0;
                clockMin++;
                DUR--;
                if(DUR == 0)
                  {
                    STEP++;
                    if(STEP > numSteps)
                      {
                        onFLG = 0;
                        last_waitTime = millis();
                        gfx->fillRect(0,90, 320, 80, BLACK);
                        gfx->setFont(u8g2_font_logisoso58_tf);
                        gfx->setTextColor(YELLOW);
                        gfx->setCursor(5,160);
                        int x = random(0,9);
                        gfx->print(OVERmessage[x]);
                        Serial.println("END");
                      }
                    else  updateLevel(STEP);
                  }
              }
            Serial.print(clockMin); Serial.print(":"); Serial.println(clockSec);
            updateTime();
          }
      }
    else
      // put to sleep if nothing happens for 60 sec (battery charging mode)
      {
        waitTime = millis();
        if((waitTime - last_waitTime) >= criterionWait)
          {
            gfx->displayOff();
            esp_deep_sleep_start();
          }
      }
  }    
  
void updateTime()
  {
    // update timer display every second
    gfx->setFont(u8g2_font_inb30_mf);
    gfx->setTextSize(1);
    gfx->setTextColor(YELLOW);
    gfx->fillRect(111, 1, 135, 38, BLUE);
    gfx->drawRect(110, 0, 137, 40, WHITE);
    gfx->setCursor(115, 35);
    if(clockMin < 10) gfx->print("0");
    gfx->print(clockMin);
    gfx->print(":");
    if(clockSec < 10) gfx->print("0");
    gfx->print(clockSec);
  }

void updateLevel(int stp)
  {
    DUR = METs[workout][stp][0].toInt();
    INTERVAL = METs[workout][stp][1];
    INC = METs[workout][stp][2];
    SPD = METs[workout][stp][3];
    INTERVAL1 = METs[workout][stp+1][1];
    INC1 = METs[workout][stp+1][2];
    SPD1 = METs[workout][stp+1][3];
    gfx->fillRect(280,0, 40, 35, BLACK);
    gfx->setTextColor(CYAN);
    gfx->setFont(u8g2_font_ncenR18_tf);
    gfx->setCursor(285,35);
    gfx->print(stp);
    gfx->fillRect(0,90, 320, 80, BLACK);
    gfx->setFont(u8g2_font_inb24_mf);
    gfx->setCursor(1,125);
    gfx->setTextColor(WHITE);
    gfx->println(INTERVAL + ": " + INC + " " + SPD);
    gfx->setTextColor(RED);
    gfx->println(INTERVAL1 + ": " + INC1 + " " + SPD1);
  }

void showMETlabel()
  {
    gfx->setCursor(15, 35);
    gfx->setTextColor(CYAN);
    gfx->setFont(u8g2_font_ncenR18_tf);
    gfx->print(LBL);
  }
    
void showTreadLabels()
  {
    gfx->setCursor(15,80);
    gfx->setTextColor(GREEN);
    gfx->setFont(u8g2_font_inb24_mf);
    gfx->println(" Min:  %  mph");
    gfx->drawFastHLine(0, 89, 320, GREEN);
  }
